@@ -2,7 +2,6 @@ triang.list <- function (object)
 {
     stopifnot(inherits(object,"deldir"))
     io <- object$ind.orig
-    pt <- object$summary$pt.type
     tlist <- prelimtlist(object)
     x <- object$summary[,"x"]
     y <- object$summary[,"y"]
@@ -40,13 +39,11 @@ triang.list <- function (object)
 		u=as.double(x),
 		v=as.double(y),
 		n=as.integer(length(x)),
-		okay=logical(1),
+		okay=integer(1),
 		PACKAGE="deldir"
 	)
-	if(tmp$okay) {
+	if(as.logical(tmp$okay)) {
                 tmp <- data.frame(ptNum=io[tlist[i,]],x=xtri[i,],y=ytri[i,])
-                if(length(pt)) tmp <- cbind(tmp[,1,drop=FALSE],
-                                            ptType=pt[tlist[i,]],tmp[,2:3])
                 if(haveZ) tmp <- cbind(tmp,z=ztri[i,])
 		K <- K+1
                 rslt[[K]] <- tmp
